from gister.utils.text import wrap


class PrintLog:

    def info(message: str):
        print(f"\033[1;34;10m{wrap(message)}\033[0m\n")

    def success(message: str):
        print(f"\033[1;32;10m{wrap(message)} \U0001F44D\033[0m\n")

    def warning(message: str):
        print(f"\033[1;33;10m{wrap(message)} \U0001F9DF\033[0m\n")

    def error(message: str):
        print(f"\033[1;31;10m{wrap(message)} \U0001F525\033[0m\n")



from pathlib import Path

from gister.errors import ContextError, FileTemplateError
from gister.log.print import PrintLog

""" Git template section
"""


def user_section(name="", email=""):
    return f'\
[user]\n\
  name="{name}"\n\
  email="{email}"'


def filter_section():
    return f'\n\
[filter "lfs"]\n\
  clean=git-lfs clean -- %f\n\
  smudge=git-lfs smudge -- %f\n\
  process=git-lfs filter-process\n\
  required=true\n'


def ssh_section():
    return f'\n\
[url "ssh://git@github.com/"]\n\
  insteadOf=https://github.com/\
  '


""" Process and Create template
"""


def process_template(template=""):
    try:
        if template == "":
            raise ContextError("Template must not empty")
        path = f"{Path.home()}/.gitconfig"
        with open(path, "w") as file:
            file.write(template)
            file.close()
            PrintLog.success("git environment switched!")

    except FileTemplateError:
        raise



import logging

from gister.parser.yamlfile import ConfigReader
from gister.context import Context
from gister.steps.gitenv import process_template, user_section, filter_section, ssh_section
from gister.log.print import PrintLog

logger = logging.getLogger(__name__)


def change_env(env):
    logger.debug("gitenv-environment: ", env)
    context = ConfigReader().read_json_configs()
    process_context(context, env)
    set_gitenv_active_environment(env)


def process_context(context, env):
    git_env = "gitenv"
    ssh = "ssh"
    parsed_context = Context(context)
    """ Configuration file example
        gitenv:
            work:
                username: "John Doe"    # required
                email: "john@john.me"   # required
                ssh: true | false       # not required
            .
            .
            .
    """
    ENV = "env"
    if parsed_context.assert_key_exists(git_env, __name__) is None:
        # if parsed_context.assert_child_key_has_value(git_env, env, __name__) is None:
        gitenv_environments = Context(parsed_context.get_formatted(git_env))
        # print("Context", gitenv_env)
        if gitenv_environments.assert_child_key_has_value(ENV, env, __name__) is None:
            """ Valid if git env config exists on file
            """
            gitenv_context = parsed_context.get_formatted(git_env)

            parse_conf = Context(gitenv_context[ENV][env])
            keys = ("username", "email")
            has_ssh_conf = {'key_in_context': 'ssh', 'has_value': "true", 'is_expected_type': bool}
            if parse_conf.assert_keys_exist(__name__, *keys) is None and parse_conf.assert_keys_have_values(__name__,
                                                                                                            *keys) is None:
                username = gitenv_context[ENV][env]["username"]
                email = gitenv_context[ENV][env]["email"]
                template = user_section(gitenv_context[ENV][env]["username"], gitenv_context[ENV][env]["email"])
                """ Validate if ssh config is on .gisterconfig file
                    If template is checked as true in yml key the git config for ssh is configured
                    Example:
                    gitenv:
                        test:
                            username: "Test username"
                            email: "test email"
                            ssh: true

                """
                if gitenv_context[ENV][env][ssh] is not None and parse_conf.assert_key_type_value(has_ssh_conf,
                                                                                                  __name__) is None:
                    if type(gitenv_context[ENV][env][ssh]) is bool and gitenv_context[ENV][env][ssh]:
                        template += ssh_section()

                template += filter_section()
                PrintLog.info("Conf:\n" f"user:{username}\nemail:{email}")

                # Create template to .gitconfig
                process_template(template)


def set_gitenv_active_environment(env):
    gister_conf = ConfigReader()
    yml_file = gister_conf.read_yml_configs()
    yml_file["gitenv"]["active"] = env
    gister_conf.write_new_yml_properties(yml_file)



from gister.errors import (ContextError, KeyNotInContextError, KeyInContextHasNoValueError)


def assert_key_exists(obj, key, caller, parent=None):
    try:
        if key not in obj:
            if parent:
                msg = (f"context[{parent!r}][{key!r}] doesn't "
                       f"exist. It must exist for {caller}.")
            else:
                msg = (
                    f"context[{key!r}] doesn't exist. "
                    f"It must exist for {caller}."
                )
            raise KeyNotInContextError(msg)
    except TypeError as err:
        if parent:
            msg = (f"context[{parent!r}] must exist, be iterable and contain "
                   f"{key!r} for {caller}. {err}")
        else:
            msg = (f"context[{key!r}] must exist and be iterable for "
                   f"{caller}. {err}")
        raise ContextError(msg) from err


def assert_key_has_value(obj, key, caller, parent=None):
    assert_key_exists(obj, key, caller, parent)
    if obj[key] is None:
        if parent:
            msg = (f"context[{parent!r}][{key!r}] must have a value for "
                   f"{caller}.")
        else:
            msg = f"context[{key!r}] must have a value for {caller}."

        raise KeyInContextHasNoValueError(msg)


def assert_key_is_truthy(obj, key, caller, parent=None):
    assert_key_exists(obj, key, caller, parent)
    if not obj[key]:
        if parent:
            msg = (f"context[{parent!r}][{key!r}] must have a value for "
                   f"{caller}.")
        else:
            msg = f"context[{key!r}] must have a value for {caller}."

        raise KeyInContextHasNoValueError(msg)



import setuptools
from setuptools import setup, find_packages
#from gister.version import __version__

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
# packages = ["gister"],
# url="https://github.com/pypa/sampleproject",
#project_urls={
#    "Bug Tracker": "https://github.com/pypa/sampleproject/issues",
#},
setuptools.setup(
    name="gister-cli",
    version="0.0.4",
    author="Josh Andres",
    author_email="josh.an.bas@gmail.com",
    description="A awesome cli project.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    package_dir={'': "gister"},
    packages=find_packages("gister"),
    python_requires=">=3.6",
    install_requires=[
        'requests',
        'ruamel.yaml==0.17.17',
        'importlib; python_version == "3.6"',
    ],
    entry_points={
        'console_scripts': [
            'gister=gister.cli:main'
        ],
    },
)
